import numpy as np


def sizeof_fmt(num, suffix='B'):
    for unit in ['', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi']:
        if abs(num) < 1024.0:
            return "%3.1f%s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, 'Yi', suffix)








def env_state_shapes(env):
    return [obs.shape for obs in env.observation_space.spaces] \
        if hasattr(env,'spaces') else [env.observation_space.shape]



import numpy as np


def image_layout(x: np.ndarray, old: str, new: str) -> np.ndarray:
    new = [old.index(char) for char in new]
    return x.transpose(new)


def hwc_to_chw(x: np.ndarray) -> np.ndarray:
    if x.ndim == 3:
        x_ = image_layout(x, 'hwc', 'chw')
    elif x.ndim == 4:
        x_ = image_layout(x, 'nhwc', 'nchw')
    else:
        raise ValueError(f"s has {x.ndim} dimensions, 3 or 4 dimensions needed")
    return x_


def chw_to_hwc(x: np.ndarray) -> np.ndarray:
    if x.ndim == 3:
        x_ = image_layout(x, 'chw', 'hwc')
    elif x.ndim == 4:
        x_ = image_layout(x, 'nchw', 'nhwc')
    else:
        raise ValueError(f"s has {x.ndim} dimensions, 3 or 4 dimensions needed")
    return x_


def s_hwc_to_chw(s):
    # TODO: HWC to CHW conversion optimized here
    # because pytorch can only deal with images in CHW format
    # we are making the optimization here to convert from HWC to CHW format.
    if isinstance(s, np.ndarray) and (s.ndim > 2):
        s = hwc_to_chw(s)
    elif isinstance(s, list):  # state is a list of states
        for i in range(len(s)):
            if isinstance(s[i], np.ndarray) and (s[i].ndim > 2):
                s[i] = hwc_to_chw(s[i])
    return s



import yaml


def config_banner(config, config_name):
    return f'----------------------------------------\n' \
           f'{config_name}\n' \
           f'----------------------------------------\n' \
           f'{yaml.dump(config, default_flow_style=False)}' \
           f'----------------------------------------\n'


def save_config(config, config_filename):
    with open(config_filename, 'w') as f:
        return yaml.dump(config, f, default_flow_style=False)


def load_config(config_filename):
    with open(config_filename, 'r') as f:
        return yaml.safe_load(f)



# navsim_util

from pathlib import Path

version_file = Path(__file__).parent.joinpath('version.txt')
if not version_file.exists():
    version_file = Path(__file__).parent.joinpath('../../version.txt')
with open(version_file, 'r') as vf:
    __version__ = vf.read().strip()

from . import image, env

__all__ = ['image', 'env', 'dict']



import cv2
import numpy as np


def isbright(image, dim=10, thresh=0.5):
    # Resize image to 10x10
    image = cv2.resize(image, (dim, dim))
    # Convert color space to LAB format and extract L channel
    L, A, B = cv2.split(cv2.cvtColor(image, cv2.COLOR_BGR2LAB))
    # Normalize L channel by dividing all pixel values with maximum pixel value
    L = L / np.max(L)
    # Return True if mean is greater than thresh else False
    return np.mean(L) > thresh



def env_info(env):
    """Prints the information about the environment

    """
    print('-----------')
    print("Env Info")
    print('-----------')
    if env.spec is not None:
        print(env.spec.id)
    print('Action Space:', env.action_space)
    # TODO: print env_config if it exists
    # print('Observation Mode:', env.obs_mode)
    # print('Gym Observation Space:', self.genv.observation_space)
    # print('Gym Observation Space Shape:', self.genv.observation_space.shape)
    print('Observation Space:', env.observation_space)
    #if hasattr(env.observation_space, 'spaces'):
    #    print('Observation Space Spaces:',
    #          [obs for obs in env.observation_space.spaces])
    #    print('Observation Space Types:', [type(obs) for obs in env.observation_space.spaces])
    print('Reward Range:', env.reward_range)
    print('Metadata:', env.metadata)
    print('--------------------------------------')


def register_with_gym(env_id: str, entry_point: str):
    """Registers the environment with gym registry

    """
    from gym.envs.registration import register, registry

    env_dict = registry.env_specs.copy()
    for env in env_dict:
        if env_id in env:
            print(f"Removing {env} from Gym registry")
            del registry.env_specs[env]

    print(f"navsim_envs: Adding {env_id} to Gym registry")
    return register(id=env_id, entry_point=entry_point)



import setuptools

with open('../version.txt', 'r') as vf:
    __version__ = vf.read().strip()

# TODO: Replace the long description comments once cleared
# with open("../README.md", "r", encoding="utf-8") as fh:
#    long_description = fh.read()
long_description = " "

setuptools.setup(
    name="ezai_util",
    version=__version__,
    author="Armando Fandango",
    author_email="armando@neurasights.com",
    description="EZAI Utilities and Tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
    ],
    include_package_data=True,
    # data_files=[('n',['n/version.txt'])],
    python_requires='>=3.8',
    install_requires=[
        'numpy',
        'opencv-python',
        'pytest',
        'click',
        #'gym>=0.18.0',
        #'gym_unity==0.27.0'
    ],
    entry_points={
        "console_scripts": [
#            "n_env_test=n_envs.env_tst:main",
        ]
    },
)



import numpy as np
def image_write(filename:str, image:np.ndarray):
    try:
        from cv2 import imwrite as imwrite
        print("using cv2 as image library")
    except ImportError as error:
        try:
            from imageio import imwrite as imwrite
            print("using imageio as image library")
        except ImportError as error:
            try:
                from matplotlib.pyplot import imsave as imwrite
                print("using matplotlib as image library")
            except ImportError as error:
                try:
                    from PIL import Image
                    print("using PIL as image library")

                    def imwrite(filename, arr):
                        im = Image.fromarray(arr)
                        im.save(filename)
                except ImportError as error:
                    def imwrite(filename=None, arr=None):
                        print("unable to load any of the following "
                              "image libraries: cv2, imageio, matplotlib, "
                              "PIL. Install one of these libraries to "
                              "save visuals.")

                    imwrite()



from .cv2_util import isbright
from .np_util import (
    image_layout, hwc_to_chw, chw_to_hwc, s_hwc_to_chw
)

__all__ = ['isbright',
           'image_layout',
           'hwc_to_chw',
           'chw_to_hwc',
           's_hwc_to_chw']